/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource;

import inti.util.NotifyingInputStream;
import inti.util.NotifyingInputStream.ReadNotifier;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class ResourceWorkingContext {
    
    MessageDigest digest;
    StringBuilder builder = new StringBuilder(128);
    NotifyingInputStream notifyingInputStream = new NotifyingInputStream();
    
    public ResourceWorkingContext() {
        try {
            digest = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException exception) {
        }
        
        notifyingInputStream.setNotifier(new ReadNotifier() {
            
            @Override
            public void read(byte[] data, int offset, int length) {
                if (length > 0) {
                    digest.update(data, offset, length);
                }
            }
            
            @Override
            public void read(int b) {
                if (b >= 0 && b <= 255) {
                    digest.update((byte) b);
                }
            }
            
            @Override
            public void close() {
            }
            
        });
    }
    
    public StringBuilder getBuilder() {
        return builder;
    }
    
    public MessageDigest getDigest() {
        return digest;
    }
    
    public NotifyingInputStream getNotifyingInputStream() {
        return notifyingInputStream;
    }
    
    public void clearBuilder() {
        builder.delete(0, builder.length());
    }
    
}