/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource.mail;

import inti.ws.spring.resource.config.ConfigParser;
import inti.ws.spring.resource.config.ConfigParserSettings;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import javax.inject.Inject;
import javax.servlet.ServletContext;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

@Component
@Scope("prototype")
public class MailsConfig implements ConfigParser<MailResource> {
    
    public static class MailBean {
        
        String content;
        String subject;
        
        public String getContent() {
            return content;
        }
        
        public void setContent(String content) {
            this.content = content;
        }
        
        public String getSubject() {
            return subject;
        }
        
        public void setSubject(String subject) {
            this.subject = subject;
        }
        
    }
    
    @Inject
    ServletContext ctx;
    protected ConfigParserSettings settings;
    
    @Override
    public Map<String, MailResource> instanceWebResources(ObjectMapper mapper, JsonNode node) throws Exception {
        Iterator<Entry<String, JsonNode>> fields = node.fields();
        Entry<String, JsonNode> field;
        Map<String, MailResource> resources = new HashMap<>();
        
        while (fields.hasNext()) {
            field = fields.next();
            resources.put(field.getKey(), parseMail(mapper.convertValue(field.getValue(), MailBean.class)));
        }
        
        return resources;
    }
    
    @Override
    public void configureWebResources(ObjectMapper mapper, JsonNode node, Map<String, MailResource> resources,
            Map<String, Map<String, MailResource>> resourceStack) throws Exception {
    }
    
    protected MailResource parseMail(MailBean mailBeans) throws Exception {
        MailResource mailResource;
        
        mailResource = new MailResource(ctx, mailBeans.getContent());
        mailResource.setSubjectTemplate(mailBeans.getSubject());
        
        return mailResource;
    }
    
    @Override
    public ConfigParserSettings getConfigParserSettings() {
        return settings;
    }
    
    @Override
    public void setConfigParserSettings(ConfigParserSettings settings) {
        this.settings = settings;
    }
    
}
