/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource.minify;

import org.springframework.stereotype.Component;

import com.google.javascript.jscomp.CompilationLevel;
import com.google.javascript.jscomp.Compiler;
import com.google.javascript.jscomp.CompilerOptions;
import com.google.javascript.jscomp.JSSourceFile;

@Component
public class JSMinifier implements Minifier {
    
    CompilerOptions options = new CompilerOptions();
    
    public JSMinifier() {
        if (System.getProperty("DEBUG_MODE") == null || !System.getProperty("DEBUG_MODE").equals("true")) {
            CompilationLevel.SIMPLE_OPTIMIZATIONS.setOptionsForCompilationLevel(options);
        }
    }
    
    @Override
    public String minify(String script) {
        Compiler compiler;
        JSSourceFile extern;
        JSSourceFile input;
        
        if (script != null) {
            compiler = new Compiler();
            extern = JSSourceFile.fromCode("externs.js", "");
            input = JSSourceFile.fromCode("input.js", script);
            
            compiler.compile(extern, input, options);
            
            return '\n' + compiler.toSource();
        }
        return null;
    }
    
}
