/**
 * Copyright 2012 the project-owners
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package inti.ws.spring.resource.template;

import inti.ws.spring.resource.WebResource;
import inti.ws.spring.resource.config.ConfigParser;
import inti.ws.spring.resource.config.ConfigParserSettings;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.inject.Inject;
import javax.servlet.ServletContext;

import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

@Component
@Scope("prototype")
public class TemplatesConfig implements ConfigParser<TemplateResource> {
    
    public static class TemplateBean {
        
        List<String> files;
        String template;
        String container;
        Map<String, Object> parameters;
        
        public String getContainer() {
            return container;
        }
        
        public void setContainer(String container) {
            this.container = container;
        }
        
        public List<String> getFiles() {
            return files;
        }
        
        public void setFiles(List<String> files) {
            this.files = files;
        }
        
        public String getTemplate() {
            return template;
        }
        
        public void setTemplate(String template) {
            this.template = template;
        }
        
        public Map<String, Object> getParameters() {
            return parameters;
        }
        
        public void setParameters(Map<String, Object> parameters) {
            this.parameters = parameters;
        }
        
    }
    
    @Inject
    ServletContext ctx;
    protected ConfigParserSettings settings;
    
    @Override
    public Map<String, TemplateResource> instanceWebResources(ObjectMapper mapper, JsonNode node) throws Exception {
        Iterator<Entry<String, JsonNode>> fields = node.fields();
        Entry<String, JsonNode> field;
        Map<String, TemplateResource> resources = new HashMap<>();
        
        while (fields.hasNext()) {
            field = fields.next();
            resources.put(field.getKey(),
                    parseTemplate(field.getKey(), mapper.convertValue(field.getValue(), TemplateBean.class)));
        }
        
        return resources;
    }
    
    @Override
    public void configureWebResources(ObjectMapper mapper, JsonNode node, Map<String, TemplateResource> resources,
            Map<String, Map<String, TemplateResource>> resourceStack) throws Exception {
    }
    
    protected TemplateResource parseTemplate(String resourceName, TemplateBean templateBean) throws Exception {
        List<WebResource> templateList;
        
        templateList = new ArrayList<>();
        
        for (String file : templateBean.getFiles()) {
            templateList.add(new WebResource(ctx, file, null, false));
        }
        
        return new TemplateResource(ctx, resourceName, templateBean.getContainer(), templateBean.getTemplate(),
                templateList, templateBean.getParameters(), null, false);
    }
    
    @Override
    public ConfigParserSettings getConfigParserSettings() {
        return settings;
    }
    
    @Override
    public void setConfigParserSettings(ConfigParserSettings settings) {
        this.settings = settings;
    }
    
}
